# Generated by Django 5.0.2 on 2024-03-21 15:11

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cjkcms", "0020_socialmediasettings_github_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="layoutsettings",
            name="navbar_color_scheme",
        ),
        migrations.AddField(
            model_name="layoutsettings",
            name="color_scheme",
            field=models.CharField(
                blank=True,
                choices=[],
                default="",
                help_text="Default light/dark/custom theme. (MD/Bootstrap only)",
                max_length=50,
                verbose_name="Color scheme",
            ),
        ),
        migrations.AddField(
            model_name="layoutsettings",
            name="light_dark_switch",
            field=models.BooleanField(
                default=False,
                help_text="Show switch to toggle light/dark theme (MD/Bootstrap only)",
                verbose_name="Light/Dark switch",
            ),
        ),
        migrations.AlterField(
            model_name="layoutsettings",
            name="frontend_theme",
            field=models.CharField(
                blank=True,
                choices=[],
                default="",
                help_text="Change the source of your Bootstrap theme.",
                max_length=50,
                verbose_name="Theme variant",
            ),
        ),
        migrations.AlterField(
            model_name="layoutsettings",
            name="navbar_collapse_mode",
            field=models.CharField(
                blank=True,
                choices=[],
                default="",
                help_text="Control on what screen sizes to show and collapse the navbar menu links.",
                max_length=50,
                verbose_name="Collapse navbar menu",
            ),
        ),
        migrations.AlterField(
            model_name="layoutsettings",
            name="navbar_format",
            field=models.CharField(
                blank=True,
                choices=[],
                default="",
                max_length=50,
                verbose_name="Navbar format",
            ),
        ),
        migrations.AlterField(
            model_name="layoutsettings",
            name="navbar_langselector",
            field=models.CharField(
                blank=True,
                choices=[],
                default=None,
                help_text="Choose lang choice selector",
                max_length=255,
                null=True,
                verbose_name="Language selector",
            ),
        ),
    ]



from django.conf import settings
from typing import Optional


class _DefaultSettings:
    CJKCMS_FONT_URL = "https://fonts.googleapis.com/css2?family=Roboto:wght@300;400;500;700&display=swap"  # noqa

    CJKCMS_FONT_FAMILY = "Roboto, sans-serif"

    CJKCMS_BRAND_LOGO_LONG = "cjkcms/images/logos/cms-logo-long.svg"
    CJKCMS_BRAND_LOGO_SQUARE = "cjkcms/images/logos/cms-logo-square.svg"
    CJKCMS_FRONTEND_BTN_SIZE_DEFAULT = ""
    CJKCMS_FRONTEND_BTN_SIZE_CHOICES = [
        ("btn-sm", "Small"),
        ("", "Default"),
        ("btn-lg", "Large"),
    ]

    CJKCMS_FRONTEND_BTN_STYLE_DEFAULT = "btn-primary"
    CJKCMS_FRONTEND_BTN_STYLE_CHOICES = [
        ("btn-primary", "Primary"),
        ("btn-secondary", "Secondary"),
        ("btn-success", "Success"),
        ("btn-danger", "Danger"),
        ("btn-warning", "Warning"),
        ("btn-info", "Info"),
        ("btn-link", "Link"),
        ("btn-light", "Light"),
        ("btn-dark", "Dark"),
        ("btn-outline-primary", "Outline Primary"),
        ("btn-outline-secondary", "Outline Secondary"),
        ("btn-outline-success", "Outline Success"),
        ("btn-outline-danger", "Outline Danger"),
        ("btn-outline-warning", "Outline Warning"),
        ("btn-outline-info", "Outline Info"),
        ("btn-outline-light", "Outline Light"),
        ("btn-outline-dark", "Outline Dark"),
    ]

    CJKCMS_FRONTEND_CAROUSEL_FX_DEFAULT = ""
    CJKCMS_FRONTEND_CAROUSEL_FX_CHOICES = [
        ("", "Slide"),
        ("carousel-fade", "Fade"),
    ]

    CJKCMS_FRONTEND_COL_SIZE_DEFAULT = ""
    CJKCMS_FRONTEND_COL_SIZE_CHOICES = [
        ("", "Automatically size"),
        ("12", "Full row"),
        ("6", "Half - 1/2 column"),
        ("4", "Thirds - 1/3 column"),
        ("8", "Thirds - 2/3 column"),
        ("3", "Quarters - 1/4 column"),
        ("9", "Quarters - 3/4 column"),
        ("2", "Sixths - 1/6 column"),
        ("10", "Sixths - 5/6 column"),
        ("1", "Twelfths - 1/12 column"),
        ("5", "Twelfths - 5/12 column"),
        ("7", "Twelfths - 7/12 column"),
        ("11", "Twelfths - 11/12 column"),
    ]

    CJKCMS_FRONTEND_COL_BREAK_DEFAULT = "md"
    CJKCMS_FRONTEND_COL_BREAK_CHOICES = [
        ("", "Always expanded"),
        ("sm", "sm - Expand on small screens (phone, 576px) and larger"),
        ("md", "md - Expand on medium screens (tablet, 768px) and larger"),
        ("lg", "lg - Expand on large screens (laptop, 992px) and larger"),
        ("xl", "xl - Expand on extra large screens (wide monitor, 1200px)"),
    ]

    CJKCMS_FRONTEND_NAVBAR_FORMAT_DEFAULT = ""
    CJKCMS_FRONTEND_NAVBAR_FORMAT_CHOICES = [
        ("", "Default Bootstrap Navbar"),
        ("cjkcms-navbar-center", "Centered logo at top"),
    ]

    CJKCMS_FRONTEND_SEARCH_FORMAT_DEFAULT = "box"
    CJKCMS_FRONTEND_SEARCH_FORMAT_CHOICES = [
        ("box", "Search box, no button"),
        ("box-button", "Search box with button"),
        ("button-popup", "Button opening search popup"),
    ]

    CJKCMS_LANGUAGE_SELECTOR_DEFAULT: Optional[str] = None
    CJKCMS_LANGUAGE_SELECTOR_CHOICES = [
        (None, "None"),
        ("cjkcms/snippets/navbar_lang_selector.html", "Menu Dropdown Selector"),
        ("cjkcms/snippets/bottom_corner_lang_selector.html", "Bottom Corner Selector"),
    ]

    CJKCMS_FRONTEND_COLOR_SCHEME_DEFAULT = "light"
    CJKCMS_FRONTEND_COLOR_SCHEME_CHOICES = [
        ("light", "Light theme"),
        ("dark", "Dark theme"),
        ("custom", "Custom from cjkcms-custom-theme.css"),
    ]

    CJKCMS_FRONTEND_NAVBAR_CLASS_DEFAULT = "bg-light"

    CJKCMS_FRONTEND_NAVBAR_COLLAPSE_MODE_DEFAULT = "navbar-expand-lg"
    CJKCMS_FRONTEND_NAVBAR_COLLAPSE_MODE_CHOICES = [
        ("", "Never show menu - Always collapse menu behind a button"),
        ("navbar-expand-sm", "sm - Show on small screens (phone size) and larger"),
        ("navbar-expand-md", "md - Show on medium screens (tablet size) and larger"),
        ("navbar-expand-lg", "lg - Show on large screens (laptop size) and larger"),
        (
            "navbar-expand-xl",
            "xl - Show on extra large screens (desktop, wide monitor)",
        ),
    ]

    CJKCMS_FRONTEND_THEME_HELP = "Change the source of your Bootstrap theme."
    CJKCMS_FRONTEND_THEME_DEFAULT = "bootstrap5"
    CJKCMS_FRONTEND_THEME_CHOICES = (
        ("bootstrap5", "Default - Bootstrap 5 (CDN)"),
        ("mdb", "MDBootstrap 5 (CDN)"),
        ("python-webpack", "Python Webpack Boilerplate by Michael Yin"),
        ("mdb.light", "MDBootstrap 5 (local)"),
        ("mdb.dark", "MDBootstrap 5 dark (local)"),
        ("bootswatch.cerulean", "Bootswatch Cerulean (CDN)"),
        ("bootswatch.cosmo", "Bootswatch Cosmo (CDN)"),
        ("bootswatch.cyborg", "Bootswatch Cyborg (CDN)"),
        ("bootswatch.darkly", "Bootswatch Darkly (CDN)"),
        ("bootswatch.flatly", "Bootswatch Flatly (CDN)"),
        ("bootswatch.journal", "Bootswatch Journal (CDN)"),
        ("bootswatch.litera", "Bootswatch Litera (CDN)"),
        ("bootswatch.lumen", "Bootswatch Lumen (CDN)"),
        ("bootswatch.lux", "Bootswatch Lux (CDN)"),
        ("bootswatch.materia", "Bootswatch Materia (CDN)"),
        ("bootswatch.minty", "Bootswatch Minty (CDN)"),
        ("bootswatch.morph", "Bootswatch Morph (CDN)"),
        ("bootswatch.pulse", "Bootswatch Pulse (CDN)"),
        ("bootswatch.quartz", "Bootswatch Quartz (CDN)"),
        ("bootswatch.sandstone", "Bootswatch Sandstone (CDN)"),
        ("bootswatch.simplex", "Bootswatch Simplex (CDN)"),
        ("bootswatch.sketchy", "Bootswatch Sketchy (CDN)"),
        ("bootswatch.slate", "Bootswatch Slate (CDN)"),
        ("bootswatch.solar", "Bootswatch Solar (CDN)"),
        ("bootswatch.spacelab", "Bootswatch Spacelab (CDN)"),
        ("bootswatch.superhero", "Bootswatch Superhero (CDN)"),
        ("bootswatch.united", "Bootswatch United (CDN)"),
        ("bootswatch.vapor", "Bootswatch vapor (CDN)"),
        ("bootswatch.yeti", "Bootswatch Yeti (CDN)"),
        ("bootswatch.zephyr", "Bootswatch Zephyr (CDN)"),
    )

    # create a list of tuples, each tuple is a (name, [css url, js url]),
    # for each theme defined in FRONTEND_THEME_CHOICES

    CJKCMS_THEME_FILES = {
        "python-webpack": ["@include", "@include"],  # this entry will be ignored
        "bootstrap5": [
            "https://cdnjs.cloudflare.com/ajax/libs/bootstrap/5.3.3/css/bootstrap.min.css",
            "https://cdnjs.cloudflare.com/ajax/libs/bootstrap/5.3.3/js/bootstrap.bundle.min.js",
        ],
        "mdb": [
            "https://cdnjs.cloudflare.com/ajax/libs/mdb-ui-kit/7.3.2/mdb.min.css",
            "https://cdnjs.cloudflare.com/ajax/libs/mdb-ui-kit/7.3.2/mdb.umd.min.js",
        ],
        # "mdb.dark": [
        #     "https://cdnjs.cloudflare.com/ajax/libs/mdb-ui-kit/7.2.0/mdb.dark.min.css",
        #     "https://cdnjs.cloudflare.com/ajax/libs/mdb-ui-kit/7.2.0/mdb.umd.min.js",
        # ],
        "mdb.light": [  # 7.3.2
            "vendor/mdb/css/mdb.min.css",
            "vendor/mdb/js/mdb.umd.min.js",
        ],
        "mdb.dark": [  # 7.3.2
            "vendor/mdb/css/mdb.dark.min.css",
            "vendor/mdb/js/mdb.umd.min.js",
        ],
        "bootswatch.cerulean": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/cerulean/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.cosmo": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/cosmo/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.cyborg": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/cyborg/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.darkly": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/darkly/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.flatly": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/flatly/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.journal": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/journal/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.litera": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/litera/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.lumen": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/lumen/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.lux": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/lux/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.materia": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/materia/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.minty": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/minty/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.morph": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/morph/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.pulse": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/pulse/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.quartz": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/quartz/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.sandstone": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/sandstone/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.simplex": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/simplex/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.sketchy": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/sketchy/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.slate": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/slate/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.solar": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/solar/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.spacelab": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/spacelab/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.superhero": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/superhero/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.united": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/united/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.vapor": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/vapor/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.yeti": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/yeti/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
        "bootswatch.zephyr": [
            "https://cdn.jsdelivr.net/npm/bootswatch@5.3.3/dist/zephyr/bootstrap.min.css",
            "https://cdn.jsdelivr.net/npm/bootstrap@5.3.3/dist/js/bootstrap.bundle.min.js",
        ],
    }

    CJKCMS_FRONTEND_TEMPLATES_BLOCKS = {
        "cardblock": [
            ("cjkcms/blocks/card_block.html", "Card"),
            ("cjkcms/blocks/card_horizontal.html", "Horizontal Card"),
            ("cjkcms/blocks/card_horizontal2.html", "Horizontal with background fill"),
            ("cjkcms/blocks/card_head.html", "Card with header"),
            ("cjkcms/blocks/card_foot.html", "Card with footer"),
            ("cjkcms/blocks/card_head_foot.html", "Card with header and footer"),
            ("cjkcms/blocks/card_blurb.html", "Blurb - rounded image and no border"),
            ("cjkcms/blocks/card_img.html", "Cover image - use image as background"),
            ("cjkcms/blocks/card_landing1.html", "Landing page style 1"),
            ("cjkcms/blocks/card_landing2.html", "Landing page style 2"),
        ],
        "cardgridblock": [
            (
                "cjkcms/blocks/cardgrid_group.html",
                "Card group - attached cards of equal size",
            ),
            (
                "cjkcms/blocks/cardgrid_deck.html",
                "Card deck - separate cards of equal size",
            ),
            (
                "cjkcms/blocks/cardgrid_columns.html",
                "Card masonry - fluid brick pattern",
            ),
        ],
        # DEPRECATED
        "pagelistblock": [
            ("cjkcms/blocks/pagelist_block.html", "General, simple list"),
            (
                "cjkcms/blocks/pagelist_list_group.html",
                "General, list group navigation panel",
            ),
            (
                "cjkcms/blocks/pagelist_toc_nextprev.html",
                "TOC navigation with prev / next",
            ),
            ("cjkcms/blocks/pagelist_article_media.html", "Article, media format"),
            (
                "cjkcms/blocks/pagelist_article_card_group.html",
                "Article, card group - attached cards of equal size",
            ),
            (
                "cjkcms/blocks/pagelist_article_card_deck.html",
                "Article, card deck - separate cards of equal size",
            ),
            (
                "cjkcms/blocks/pagelist_article_card_columns.html",
                "Article, card masonry - fluid brick pattern",
            ),
        ],
        # DEPRECATED
        "pagepreviewblock": [
            ("cjkcms/blocks/pagepreview_card.html", "Card"),
        ],
        "quoteblock": [
            ("cjkcms/blocks/quote_block.html", "Simple blockquote"),
            ("cjkcms/blocks/quote_block_leftbar.html", "Quote with left bar"),
            ("cjkcms/blocks/quote_block_start_end_quote.html", "With start/end quote"),
        ],
        # templates that are available for all block types
        "*": [
            ("", "Default"),
        ],
    }

    CJKCMS_FRONTEND_TEMPLATES_PAGES = {
        # templates that are available for all page types
        "*": [
            ("", "Default"),
            ("cjkcms/pages/web_page.html", "Web page showing title and cover image"),
            (
                "cjkcms/pages/web_page_notitle.html",
                "Web page without title and cover image",
            ),
            ("cjkcms/pages/base.html", "Blank page - no navbar or footer"),
        ],
    }

    CJKCMS_BANNER = None
    CJKCMS_BANNER_BACKGROUND = "#f00"
    CJKCMS_BANNER_TEXT_COLOR = "#fff"

    CJKCMS_BASE_TEMPLATE_HELP = (
        "Base template used by CMS pages, defaults to CMS built-in one"
    )
    CJKCMS_BASE_TEMPLATE_DEFAULT = "cjkcms/pages/base.html"
    CJKCMS_AUTH_VISIBILITY_DEFAULT = "all"
    CJKCMS_AUTH_VISIBILITY_CHOICES = (
        ("all", "Default (Everyone)"),
        ("non-auth-only", "Not logged in only"),
        ("auth-only", "Logged in only"),
        ("include-groups", "Visible for selected groups"),
        ("exclude-groups", "Hidden for selected groups"),
        ("hidden", "Hidden for all"),
    )

    CJKCMS_HIGHLIGHT_COLORS = (
        ("primary", "Dark Blue"),
        ("secondary", "Gray"),
        ("success", "Green"),
        ("danger", "Red"),
        ("warning", "Yellow"),
        ("info", "Light Blue"),
        ("light", "Light"),
        ("dark", "Dark"),
    )
    CJKCMS_HIGHLIGHT_DEFAULT_COLOR = "none"

    FONT_AWESOME_ICONS = (
        ("fa-solid fa-circle-check", "Checkmark"),
        ("fa-solid fa-scissors", "Scissors"),
        ("fa-solid fa-pencil", "Pencil"),
        ("fa-solid fa-question", "Question mark"),
        ("fa-solid fa-globe", "Globe"),
        ("fa-regular fa-folder", "Folder"),
        ("fa-solid fa-triangle-exclamation", "Warning Triangle"),
    )
    FONT_AWESOME_ICONS_DEFAULT = "fa-solid fa-circle-check"

    BANNER = None
    BANNER_BACKGROUND = "#f00"
    BANNER_TEXT_COLOR = "#fff"

    CJKCMS_RICHTEXT_FEATURES = {
        "default": [
            "h2",
            "h3",
            "h4",
            "bold",
            "italic",
            "link",
            "ol",
            "ul",
            "hr",
            "blockquote",
            "image",
            "centre-align",
            "left-align",
            "right-align",
        ],
        "full": [
            "h2",
            "h3",
            "h4",
            "h5",
            "h6",
            "bold",
            "italic",
            "underline",
            "ol",
            "ul",
            "larger",
            "smaller",
            "superscript",
            "subscript",
            "strikethrough",
            "link",
            "hr",
            "code",
            "document-link",
            "blockquote",
            "image",
            "embed",
            "centre-align",
            "left-align",
            "right-align",
        ],
        "minimal": ["bold", "italic", "link"],
    }

    CJKCMS_SOC_MEDIA_TEMPLATE = "cjkcms/snippets/social_media.html"
    CJKCMS_SOC_LOCATION_DEFAULT = "none"
    CJKCMS_SOC_LOCATION_CHOICES = [
        ("left", "left"),
        ("right", "right"),
        ("menu", "menu"),
        ("none", "none"),
    ]

    CJKCMS_BUTTON_SIZE_DEFAULT = "btn-sm"
    CJKCMS_BUTTON_SIZE_CHOICES = [
        ("btn-sm", "Small"),
        ("btn-md", "Medium"),
        ("btn-lg", "Large"),
    ]

    CJKCMS_BUTTON_COLOR_DEFAULT = "btn-primary"
    CJKCMS_BUTTON_COLOR_CHOICES = [
        ("btn-primary", "Primary"),
        ("btn-outline-primary", "Primary Outline"),
        ("btn-secondary", "Secondary"),
        ("btn-outline-secondary", "Secondary Outline"),
        ("btn-success", "Success"),
        ("btn-outline-success", "Success Outline"),
        ("btn-danger", "Danger"),
        ("btn-outline-danger", "Danger Outline"),
        ("btn-warning", "Warning"),
        ("btn-outline-warning", "Warning Outline"),
        ("btn-info", "Info"),
        ("btn-outline-info", "Info Outline"),
        ("btn-light", "Light"),
        ("btn-outline-light", "Light Outline"),
        ("btn-dark", "Dark"),
        ("btn-outline-dark", "Dark Outline"),
        ("btn-link", "Link"),
        ("btn-outline-link", "Link Outline"),
    ]

    CJKCMS_VERSION_MONITOR_TOKEN = ""  # blank token = disabled version monitor api
    CJKCMS_VERSION_MONITOR_ALLOWED_DOMAINS = []  # disallow from any domain by default

    def __getattribute__(self, attr: str):
        # First load from Django settings.
        # If it does not exist, load from _DefaultSettings.
        try:
            return getattr(settings, attr)
        except AttributeError:
            return super().__getattribute__(attr)


cms_settings = _DefaultSettings()



# Generated by Django 5.0.2 on 2024-02-18 13:58

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cjkcms", "0017_layoutsettings_default_seo_image"),
    ]

    operations = [
        migrations.AddField(
            model_name="layoutsettings",
            name="search_format",
            field=models.CharField(
                blank=True,
                choices=[],
                default="",
                max_length=50,
                verbose_name="Search format",
            ),
        ),
    ]



# Generated by Django 4.2.5 on 2023-09-30 21:11

import cjkcms.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cjkcms", "0014_navbar_alignment"),
    ]

    operations = [
        migrations.CreateModel(
            name="EventCalendar",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255, verbose_name="Name")),
                (
                    "custom_css_class",
                    models.CharField(
                        blank=True, max_length=255, verbose_name="Custom CSS Class"
                    ),
                ),
                (
                    "custom_id",
                    models.CharField(
                        blank=True, max_length=255, verbose_name="Custom ID"
                    ),
                ),
                (
                    "events",
                    cjkcms.fields.CjkcmsStreamField(
                        use_json_field=True, verbose_name="Events"
                    ),
                ),
            ],
            options={
                "verbose_name": "Event calendar",
            },
        ),
    ]



# Generated by Django 5.0.2 on 2024-02-18 22:01

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cjkcms", "0018_layoutsettings_search_format"),
    ]

    operations = [
        migrations.AddField(
            model_name="layoutsettings",
            name="searchbox_input_class",
            field=models.CharField(
                blank=True,
                default="border-secondary mb-0",
                help_text="Classes applied to searchbox input",
                max_length=255,
                verbose_name="Searchbox input class",
            ),
        ),
        migrations.AddField(
            model_name="layoutsettings",
            name="searchbutton_class",
            field=models.CharField(
                blank=True,
                default="btn btn-outline-secondary",
                help_text="Classes applied to search button. Add e.g. `fas fa-search` for icon.",
                max_length=255,
                verbose_name="Search button (with text) class",
            ),
        ),
        migrations.AddField(
            model_name="layoutsettings",
            name="searchbutton_label",
            field=models.CharField(
                blank=True,
                default="Search",
                help_text="Leave empty when using icon-only button. Otherwise, add button label.",
                max_length=255,
                verbose_name="Search button label",
            ),
        ),
    ]



from cjkcms.blocks.base_blocks import BaseBlock
from wagtail import blocks
from wagtail.documents.blocks import DocumentChooserBlock
from wagtail.images.blocks import ImageChooserBlock
from django.utils.translation import gettext_lazy as _


class EventPresentationBlock(BaseBlock):
    """
    Project-tailored content block to add to BODY in each page.
    This serves as a sample block showing devs how to develop and add their own
    custom blocks to the page content (and/or layout)
    """

    title = blocks.CharBlock(
        required=False,
        max_length=255,
        label=_("Title"),
    )

    start_time = blocks.TimeBlock(
        required=False,
        label=_("Start time"),
        help_text=_("Format: HH:MM, local time"),
    )

    end_time = blocks.TimeBlock(
        required=False,
        label=_("End time"),
        help_text=_("Format: HH:MM, local time"),
    )

    description = blocks.RichTextBlock(
        features=["bold", "italic", "ol", "ul", "hr", "link", "document-link", "image"],
        label=_("Description"),
        help_text=_("Optional, short description of the presentation"),
        required=False,
    )

    image = ImageChooserBlock(
        required=False,
        max_length=255,
        label=_("Image"),
        help_text=_("Logo or image"),
    )

    document = DocumentChooserBlock(
        required=False,
        label=_("Presentation PDF"),
    )

    video_id = blocks.CharBlock(
        max_length=255,
        label=_("YouTube video ID"),
        required=False,
        help_text="Insert YouTube video URL here",
    )

    class Meta:
        # note - in this example we are not adding any html block,
        # you need to add one before you can display your new block
        template = "home/blocks/event_presentation.html"
        icon = "view"
        label = "Event Presentation"
        label_format = _("{title} (Event Presentation)")


# add your own content block
PROJECT_CONTENT_STREAMBLOCKS = [
    ("event_presentation", EventPresentationBlock()),
]

# add your own layout blocks
PROJECT_LAYOUT_STREAMBLOCKS = [
    # ("custom_block", CustomBlock()),
]



import json
import unittest
from unittest.mock import patch, Mock
from urllib.error import URLError
from datetime import timedelta, datetime

from cjkcms.finders.oembed import OEmbedFinderWithReferer
from wagtail.embeds.exceptions import EmbedNotFoundException


class TestOEmbedFinderWithReferer(unittest.TestCase):
    def setUp(self):
        self.finder = OEmbedFinderWithReferer()
        self.url = "http://vimeo.com/video"
        self.endpoint = "http://oembed.com/provider"
        self.oembed_response = {
            "type": "video",
            "html": "<iframe></iframe>",
            "url": "http://example.com/embedded",
            "title": "Example Video",
            "author_name": "Author",
            "provider_name": "Provider",
            "thumbnail_url": "http://vimeo.com/thumbnail.jpg",
            "width": 640,
            "height": 480,
            "cache_age": 3600,
        }

    @patch("urllib.request.urlopen")
    @patch("django.utils.timezone.now")
    def test_find_embed_success(self, mock_now, mock_urlopen):
        # Set a fixed time for the duration of this test
        fixed_time = datetime(2024, 1, 1, 12, 0, 0)
        mock_now.return_value = fixed_time

        # Mocking the response from urlopen
        mock_response = Mock()
        mock_response.read.return_value = json.dumps(self.oembed_response).encode(
            "utf-8"
        )
        mock_urlopen.return_value = mock_response

        # Mocking the _get_endpoint method to return a predefined endpoint
        self.finder._get_endpoint = Mock(return_value=self.endpoint)

        result = self.finder.find_embed(self.url)
        expected_cache_until = fixed_time + timedelta(seconds=3600)
        expected_result = {
            "title": "Example Video",
            "author_name": "Author",
            "provider_name": "Provider",
            "type": "video",
            "thumbnail_url": "http://vimeo.com/thumbnail.jpg",
            "width": 640,
            "height": 480,
            "html": '<iframe referrerpolicy="origin"></iframe>',
            "cache_until": expected_cache_until,
        }
        self.assertEqual(result, expected_result)

    @patch("urllib.request.urlopen")
    def test_find_embed_urlerror(self, mock_urlopen):
        mock_urlopen.side_effect = URLError("Test Error")
        self.finder._get_endpoint = Mock(return_value=self.endpoint)

        with self.assertRaises(EmbedNotFoundException):
            self.finder.find_embed(self.url)



from datetime import datetime, timedelta, timezone
from wagtail.test.utils import WagtailPageTests
from django.contrib.auth.models import User, Group

from wagtail.models import Page
from cjkcms.models.cms_models import ArticlePage
import uuid


class AdvSettingsTests(WagtailPageTests):
    """Tests AdvSettings class functionality: custom id, customs class,
    visibility options, start/end visible.
    """

    def setUp(self):
        # create a test user
        test_user1 = User.objects.create_user(username="testuser1", password="12345")
        test_user1.save()

        self.group22 = Group(name="Group22")
        self.group22.save()
        test_user2 = User.objects.create_user(username="testuser2", password="12345")
        test_user2.groups.add(self.group22)
        test_user2.save()

        self.group23 = Group(name="Group23")
        self.group23.save()
        test_user3 = User.objects.create_user(username="testuser3", password="12345")
        test_user3.groups.add(self.group23)
        test_user3.save()

        test_user4 = User.objects.create_user(username="testuser4", password="12345")
        test_user4.groups.add(self.group22)
        test_user4.groups.add(self.group23)
        test_user4.save()

        self.in_the_past = datetime.now(tz=timezone.utc) - timedelta(days=1)
        self.in_the_future = datetime.now(tz=timezone.utc) + timedelta(days=1)

    def createArticlePage(self, title: str, settings: dict):
        # Get the HomePage
        home_page = Page.objects.get(path="00010001")

        body = [
            {
                "type": "quote",
                "value": {
                    "settings": settings,
                    "text": "Quotably interesting",
                    "author": "By Mr Nobody",
                },
                "id": str(uuid.uuid1()),
            }
        ]

        article_page = ArticlePage(title=title, body=body)

        home_page.add_child(instance=article_page)
        article_page.save_revision().publish()

        return article_page

    def test_visible_from_to(self):
        settings = {
            "visible_from": self.in_the_past.isoformat(),
            "visible_to": self.in_the_future.isoformat(),
        }

        self.createArticlePage(title="Test Article", settings=settings)

        ap = ArticlePage.objects.get(slug="test-article")

        response = self.client.get(ap.url)
        self.assertContains(response, "Quotably interesting", status_code=200)

    def test_not_visible_from(self):
        settings = {
            "visible_from": self.in_the_future.isoformat(),
        }

        self.createArticlePage(title="Test Article2", settings=settings)

        ap = ArticlePage.objects.get(slug="test-article2")

        response = self.client.get(ap.url)
        self.assertNotContains(response, "Quotably interesting", status_code=200)

    def test_not_visible_to(self):
        settings = {
            "visible_to": self.in_the_past.isoformat(),
        }

        self.createArticlePage(title="Test Article3", settings=settings)

        ap = ArticlePage.objects.get(slug="test-article3")

        response = self.client.get(ap.url)
        self.assertNotContains(response, "Quotably interesting", status_code=200)

    def test_visible_to_loggedin(self):
        settings = {
            "visibility": "auth-only",
            "visibility_groups": "",
        }

        self.createArticlePage(title="Test Article4", settings=settings)

        ap = ArticlePage.objects.get(slug="test-article4")

        response = self.client.get(ap.url)
        self.assertNotContains(response, "Quotably interesting", status_code=200)

        self.client.login(username="testuser1", password="12345")

        response = self.client.get(ap.url)
        # print(response.content)
        self.assertContains(response, "Quotably interesting", status_code=200)

    def test_invisible_to_loggedin(self):
        settings = {
            "visibility": "non-auth-only",
            "visibility_groups": "",
        }

        self.createArticlePage(title="Test Article5", settings=settings)

        ap = ArticlePage.objects.get(slug="test-article5")

        response = self.client.get(ap.url)
        self.assertContains(response, "Quotably interesting", status_code=200)

        self.client.login(username="testuser1", password="12345")

        ap = ArticlePage.objects.get(slug="test-article5")
        response = self.client.get(ap.url)
        print(response.content)

        # @TODO: improve the test
        # note: the page meta description will contain the "Quotably interesting",
        # because the "body_preview" call gets the content rendered without context,
        # and so it sees the page as not-logged-in

        self.assertContains(
            response, '<div class="block-quote"></div>', status_code=200
        )

    def test_visible_to_specific_group(self):
        # sourcery skip: extract-duplicate-method
        settings = {
            "visibility": "include-groups",
            "visibility_groups": "Group22,Group23",
        }

        self.createArticlePage(title="Test Article6", settings=settings)

        ap = ArticlePage.objects.get(slug="test-article6")

        response = self.client.get(ap.url)
        self.assertNotContains(response, "Quotably interesting", status_code=200)

        # wrong group
        self.client.login(username="testuser1", password="12345")
        response = self.client.get(ap.url)
        self.assertNotContains(response, "Quotably interesting", status_code=200)

        self.client.logout()
        self.client.login(username="testuser2", password="12345")
        response = self.client.get(ap.url)
        self.assertContains(response, "Quotably interesting", status_code=200)

        self.client.logout()
        self.client.login(username="testuser4", password="12345")
        response = self.client.get(ap.url)
        self.assertContains(response, "Quotably interesting", status_code=200)

    def test_invisible_to_specific_group(self):
        # sourcery skip: extract-duplicate-method
        settings = {
            "visibility": "exclude-groups",
            "visibility_groups": "Group22",
        }

        self.createArticlePage(title="Test Article6", settings=settings)

        ap = ArticlePage.objects.get(slug="test-article6")

        response = self.client.get(ap.url)
        self.assertNotContains(response, "Quotably interesting", status_code=200)

        # wrong group
        self.client.login(username="testuser1", password="12345")
        response = self.client.get(ap.url)
        self.assertContains(response, "Quotably interesting", status_code=200)

        self.client.logout()
        self.client.login(username="testuser2", password="12345")
        response = self.client.get(ap.url)
        self.assertNotContains(response, "Quotably interesting", status_code=200)



# Generated by Django 5.0.2 on 2024-03-11 19:20

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("cjkcms", "0019_layoutsettings_searchbox_input_class_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="socialmediasettings",
            name="github",
            field=models.URLField(
                blank=True, help_text="Your GitHub page URL", verbose_name="GitHub"
            ),
        ),
        migrations.AddField(
            model_name="socialmediasettings",
            name="pinterset",
            field=models.URLField(
                blank=True,
                help_text="Your Pinterest page URL",
                verbose_name="Pinterest",
            ),
        ),
        migrations.AddField(
            model_name="socialmediasettings",
            name="tiktok",
            field=models.URLField(
                blank=True, help_text="Your TikTok account URL", verbose_name="TikTok"
            ),
        ),
    ]



from .snippet_models import Navbar, EventCalendar
from wagtail.snippets.views.snippets import SnippetViewSet


class NavbarSnippet(SnippetViewSet):
    model = Navbar
    menu_label = "Navigation"
    menu_icon = "link"  # change as required
    add_to_admin_menu = True
    list_display = (
        "name",
        "custom_css_class",
        "custom_id",
    )
    search_fields = [
        "name",
    ]


class EventCalendarSnippet(SnippetViewSet):
    model = EventCalendar
    menu_label = "Public Events"
    menu_icon = "calendar"  # change as required
    # add_to_admin_menu = True
    list_display = ("name",)
    search_fields = [
        "name",
    ]



# Generated by Django 4.2.2 on 2023-07-02 17:30

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("cjkcms", "0010_filmstrip_filmpanel"),
    ]

    operations = [
        migrations.AddField(
            model_name="cjkcmspage",
            name="related_classifier_term",
            field=models.ForeignKey(
                blank=True,
                help_text="When getting related pages, pages with this term will be weighted over other classifier terms. By default, pages with the greatest number of classifiers in common are ranked highest.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="cjkcms.classifierterm",
                verbose_name="Preferred related classifier term",
            ),
        ),
        migrations.AddField(
            model_name="cjkcmspage",
            name="related_num",
            field=models.PositiveIntegerField(
                default=3, verbose_name="Number of related pages to show"
            ),
        ),
        migrations.AddField(
            model_name="cjkcmspage",
            name="related_show",
            field=models.BooleanField(
                default=False, verbose_name="Show list of related pages"
            ),
        ),
    ]



"""
Settings for running tests
"""
import os

BASE_DIR = os.path.abspath(os.path.dirname(__file__))
BASE_URL = "http://localhost"

DEBUG = True
SECRET_KEY = "mysecretkey4testing"
ALLOWED_HOSTS = ["*"]
INSTALLED_APPS = [
    "cjkcms.tests.testapp.apps.TestAppConfig",  # replace `myapp` with your app name
    # --- wagtail-cjkcms + requirements #
    "cjkcms",
    "wagtailseo",
    "wagtailcache",
    "wagtail.contrib.table_block",
    "wagtail.contrib.settings",
    "wagtail_color_panel",
    "django_bootstrap5",
    # --- end-wagtail-cjkcms #
    "wagtail.contrib.forms",
    "wagtail.contrib.redirects",
    "wagtail.contrib.sitemaps",
    "django.contrib.sitemaps",
    "wagtail.embeds",
    "wagtail.sites",
    "wagtail.users",
    "wagtail.snippets",
    "wagtail.documents",
    "wagtail.images",
    "wagtail.search",
    "wagtail.admin",
    "wagtail",
    "taggit",
    "modelcluster",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
]
MIDDLEWARE = [
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "django.middleware.security.SecurityMiddleware",
    "wagtail.contrib.redirects.middleware.RedirectMiddleware",
]
ROOT_URLCONF = "cjkcms.tests.urls"
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [
            os.path.join(BASE_DIR, "templates"),
        ],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]
WAGTAILADMIN_BASE_URL = BASE_URL
STATIC_ROOT = os.path.join(BASE_DIR, "static")
STATIC_URL = "/static/"
MEDIA_ROOT = os.path.join(BASE_DIR, "media")
MEDIA_URL = "/media/"
DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
        # "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}
LANGUAGE_CODE = "en-us"
TIME_ZONE = "UTC"
USE_I18N = True
USE_TZ = True



# Generated by Django 4.1.4 on 2022-12-28 06:28

import cjkcms.fields
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("wagtailcore", "0040_page_draft_title"),
        ("cjkcms", "0004_layoutsettings_articles_date_format_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="ProjectArticleIndexPage",
            fields=[
                (
                    "cjkcmspage_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="cjkcms.cjkcmspage",
                    ),
                ),
                (
                    "show_images",
                    models.BooleanField(default=True, verbose_name="Show images"),
                ),
                ("show_captions", models.BooleanField(default=True)),
                (
                    "show_meta",
                    models.BooleanField(
                        default=True, verbose_name="Show author and date info"
                    ),
                ),
                (
                    "show_preview_text",
                    models.BooleanField(default=True, verbose_name="Show preview text"),
                ),
                (
                    "body",
                    cjkcms.fields.CjkcmsStreamField(
                        blank=True, null=True, use_json_field=True
                    ),
                ),
            ],
            options={
                "verbose_name": "{{ uf_project }} Article Landing Page",
            },
            bases=("cjkcms.cjkcmspage",),
        ),
        migrations.CreateModel(
            name="ProjectWebPage",
            fields=[
                (
                    "cjkcmspage_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="cjkcms.cjkcmspage",
                    ),
                ),
                (
                    "body",
                    cjkcms.fields.CjkcmsStreamField(
                        blank=True, null=True, use_json_field=True
                    ),
                ),
            ],
            options={
                "verbose_name": "{{ uf_project }} Web Page",
            },
            bases=("cjkcms.cjkcmspage",),
        ),
        migrations.CreateModel(
            name="ProjectArticlePage",
            fields=[
                (
                    "cjkcmspage_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="cjkcms.cjkcmspage",
                    ),
                ),
                (
                    "caption",
                    models.CharField(
                        blank=True, max_length=255, verbose_name="Caption"
                    ),
                ),
                (
                    "author_display",
                    models.CharField(
                        blank=True,
                        help_text="Override how the author’s name displays on this article.",
                        max_length=255,
                        verbose_name="Display author as",
                    ),
                ),
                (
                    "date_display",
                    models.DateField(
                        blank=True, null=True, verbose_name="Display publish date"
                    ),
                ),
                (
                    "body",
                    cjkcms.fields.CjkcmsStreamField(
                        blank=True, null=True, use_json_field=True
                    ),
                ),
                (
                    "author",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Author",
                    ),
                ),
            ],
            options={
                "verbose_name": "{{ uf_project }} Article",
                "ordering": ["-first_published_at"],
            },
            bases=("cjkcms.cjkcmspage",),
        ),
    ]



from django.urls import include, path
from django.conf.urls.static import static
from django.contrib import admin
from django.conf import settings

from wagtail import urls as wagtail_urls
from cjkcms import urls as cjkcms_urls
from wagtail.admin import urls as wagtailadmin_urls
from wagtail.documents import urls as wagtaildocs_urls


urlpatterns = [
    path("django-admin/", admin.site.urls),
    path("admin/", include(wagtailadmin_urls)),
    path("documents/", include(wagtaildocs_urls)),
    path("", include(cjkcms_urls)),
    path("", include(wagtail_urls)),
]


if settings.DEBUG:
    from django.contrib.staticfiles.urls import staticfiles_urlpatterns

    # Serve static and media files from development server
    urlpatterns += staticfiles_urlpatterns()
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



from cjkcms.forms import SearchForm
from cjkcms.models import (
    GeneralSettings,
    LayoutSettings,
)

from django.http import Http404, HttpResponsePermanentRedirect
from django.contrib.contenttypes.models import ContentType
from django.core.paginator import Paginator, InvalidPage, EmptyPage, PageNotAnInteger
from django.shortcuts import render
from wagtail.models import Page, get_page_models

# from coderedcms.importexport import convert_csv_to_json, import_pages, ImportPagesFromCSVFileForm
from cjkcms.templatetags.cjkcms_tags import get_name_of_class

from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from django.conf import settings
from django.http import JsonResponse
import django
from wagtail import __version__ as wagtail_version
from cjkcms import __version__ as cjkcms_version
import sys


def search(request):
    """
    Searches pages across the entire site.
    """
    search_form = SearchForm(request.GET)
    pagetypes = []
    results = None
    results_paginated = None

    if search_form.is_valid():
        search_query = search_form.cleaned_data["s"]
        search_model = search_form.cleaned_data["t"]

        # get all page models
        pagemodels = sorted(get_page_models(), key=get_name_of_class)
        # filter based on is search_filterable
        for model in pagemodels:
            if hasattr(model, "search_filterable") and model.search_filterable:
                pagetypes.append(model)

        results = Page.objects.live()
        if search_model:
            try:
                # If provided a model name, try to get it
                model = ContentType.objects.get(model=search_model).model_class()
                results = results.type(model)
            except ContentType.DoesNotExist:
                # Maintain existing behavior of only returning objects if the page type is real
                results = None

        # get and paginate results
        if results:
            results = results.search(search_query)
            paginator = Paginator(
                results, GeneralSettings.for_request(request).search_num_results
            )
            page = request.GET.get("p", 1)
            try:
                results_paginated = paginator.page(page)
            except PageNotAnInteger:
                results_paginated = paginator.page(1)
            except EmptyPage:
                results_paginated = paginator.page(1)
            except InvalidPage:
                results_paginated = paginator.page(1)

    # Render template
    return render(
        request,
        "cjkcms/pages/search.html",
        {
            "request": request,
            "pagetypes": pagetypes,
            "form": search_form,
            "results": results,
            "results_paginated": results_paginated,
        },
    )


def favicon(request):
    if icon := LayoutSettings.for_request(request).favicon:
        return HttpResponsePermanentRedirect(icon.get_rendition("original").url)
    raise Http404()


def robots(request):
    return render(request, "cjkcms/robots.txt", content_type="text/plain")


class VersionView(APIView):
    def get(self, request, token):
        monitor_token = settings.CJKCMS_VERSION_MONITOR_TOKEN
        allowed_domains = settings.CJKCMS_VERSION_MONITOR_ALLOWED_DOMAINS

        host = request.META.get("HTTP_HOST")

        token_ok = len(token) < 12 or token != monitor_token
        domain_ok = allowed_domains = ["*"] or host in allowed_domains

        # minimum required token length is 12 characters
        # this also prevents sites from reporting when default
        # empty token has not been replaced in local config with a proper one

        if token_ok and domain_ok:
            return JsonResponse(
                {"error": "Forbidden."}, status=status.HTTP_403_FORBIDDEN
            )

        data = {
            "python": sys.version,
            "django": django.get_version(),
            "wagtail": wagtail_version,
            "cjkcms": cjkcms_version,
        }
        return Response(data)
